/*
 * Interface summary generation.
 *
 * Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include "summary.h"

void show_interface(struct interface *iface)
{
  printf("Interface: %s\n", iface->name);
  show_includes(iface->includes);
  show_attributes(iface->attributes);
  show_signatures(iface->signatures);
}

void show_includes(struct include *inc)
{
  if (inc == NULL)
    return;
  show_includes(inc->tail);
  printf("Include: %s\n", inc->filename);
}

void show_signatures(struct signature *sig)
{
  if (sig == NULL)
    return;
  show_signature(sig);
  show_signatures(sig->tail);
}

void show_signature(struct signature *sig)
{
  printf("Signature: %s %s\n", sig->qualifier, sig->operation);
  show_attributes(sig->attributes);
  show_parameters(sig->parameters);
}

void show_attributes(struct attribute *attr)
{
  if (attr == NULL)
    return;
  show_attribute(attr);
  show_attributes(attr->tail);
}

void show_attribute(struct attribute *attr)
{
  printf("Attribute: %s", attr->attribute);
  show_identifiers(attr->identifiers);
  printf("\n");
}

void show_parameters(struct parameter *param)
{
  if (param == NULL)
    return;
  show_parameter(param);
  show_parameters(param->tail);
}

void show_parameter(struct parameter *param)
{
  char *type;

  printf("Parameter: %s%s", param->specifier & IN_PARAMETER ? "IN" : "",
                            param->specifier & OUT_PARAMETER ? "OUT" : "");

  /* Items must have their type identifiers restored. */

  if (param->cls & ITEM_CLASS)
  {
    type = param->cls == FPAGE_ITEM ? "fpage" : "cap";
    printf(" %s", type);
  }

  show_identifiers(param->identifiers);
  printf("\n");
}

void show_identifiers(struct identifier *ident)
{
  while (ident != NULL)
  {
    printf(" %s", ident->identifier);
    ident = ident->tail;
  }
}
